from unittest.mock import MagicMock, patch
from splent_io.splent_feature_auth.services import AuthenticationService

@patch("splent_io.splent_feature_auth.services.login_user")
def test_authenticate_valid_user(mock_login_user):
    mock_user = MagicMock()
    mock_user.check_password.return_value = True

    repo = MagicMock()
    repo.get_by_email.return_value = mock_user

    service = AuthenticationService(user_repository=repo)
    result = service.login("user@example.com", "1234")

    assert result is True
    mock_login_user.assert_called_once_with(mock_user, remember=True)



from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.common.exceptions import NoSuchElementException
import time

from splent_framework.environment.host import get_host_for_selenium_testing
from splent_framework.selenium.common import initialize_driver, close_driver


def test_login_and_check_element():

    driver = initialize_driver()

    try:
        host = get_host_for_selenium_testing()

        # Open the login page
        driver.get(f"{host}/login")

        # Wait a little while to make sure the page has loaded completely
        time.sleep(4)

        # Find the username and password field and enter the values
        email_field = driver.find_element(By.NAME, "email")
        password_field = driver.find_element(By.NAME, "password")

        email_field.send_keys("user1@example.com")
        password_field.send_keys("1234")

        # Send the form
        password_field.send_keys(Keys.RETURN)

        # Wait a little while to ensure that the action has been completed
        time.sleep(4)

        try:

            driver.find_element(
                By.XPATH,
                "//h1[contains(@class, 'h2 mb-3') and contains(., 'Latest datasets')]",
            )
            print("Test passed!")

        except NoSuchElementException:
            raise AssertionError("Test failed!")

    finally:

        # Close the browser
        close_driver(driver)


# Call the test function
test_login_and_check_element()



from splent_framework.hooks.template_hooks import register_template_hook
from flask import render_template


def login_signup_items():
    return render_template("hooks/anonymous_sidebar_items.html")

def logout_item():
    return render_template("hooks/authenticated_sidebar_items.html")

register_template_hook("layout.anonymous_sidebar", login_signup_items)
register_template_hook("layout.authenticated_sidebar", logout_item)


from flask_wtf import FlaskForm
from wtforms import StringField, SubmitField, PasswordField, BooleanField
from wtforms.validators import DataRequired, Email, Length


class SignupForm(FlaskForm):
    name = StringField("Name", validators=[DataRequired(), Length(max=100)])
    surname = StringField(
        "Surname", validators=[DataRequired(), Length(max=100)]
    )
    password = PasswordField("Password", validators=[DataRequired()])
    email = StringField("Email", validators=[DataRequired(), Email()])
    submit = SubmitField("Submit")


class LoginForm(FlaskForm):
    email = StringField("Email", validators=[DataRequired()])
    password = PasswordField("Password", validators=[DataRequired()])
    remember_me = BooleanField("Remember me")
    submit = SubmitField("Login")



"""splent_feature_auth

Revision ID: 990210b54b52
Revises: 
Create Date: 2026-03-21 18:46:40.333760

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '990210b54b52'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('user',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('email', sa.String(length=256), nullable=True),
    sa.Column('password', sa.String(length=256), nullable=False),
    sa.Column('created_at', sa.DateTime(), nullable=False),
    sa.Column('active', sa.Boolean(), nullable=False),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('email')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('user')
    # ### end Alembic commands ###



from splent_io.splent_feature_auth.models import User
from splent_framework.repositories.BaseRepository import BaseRepository


class UserRepository(BaseRepository):
    def __init__(self):
        super().__init__(User)

    def create(self, commit: bool = True, **kwargs):
        password = kwargs.pop("password")
        instance = self.model(**kwargs)
        instance.set_password(password)
        self.session.add(instance)
        if commit:
            self.session.commit()
        else:
            self.session.flush()
        return instance

    def get_by_email(self, email: str, active: bool = True):
        return self.model.query.filter_by(email=email, active=active).first()



"""splent_feature_auth

Revision ID: 5ad9fd0688b1
Revises: a1fb31611281
Create Date: 2026-03-21 18:54:00.476107

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '5ad9fd0688b1'
down_revision = 'a1fb31611281'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



from splent_io.splent_feature_auth.models import User
from splent_io.splent_feature_profile.models import UserProfile
from splent_framework.seeders.BaseSeeder import BaseSeeder


class AuthSeeder(BaseSeeder):

    priority = 1  # Higher priority

    def run(self):

        # Seeding users
        users = [
            User(email="user1@example.com", password="1234"),
            User(email="user2@example.com", password="1234"),
        ]

        # Inserted users with their assigned IDs are returned by `self.seed`.
        seeded_users = self.seed(users)

        # Create profiles for each user inserted.
        user_profiles = []
        names = [("John", "Doe"), ("Jane", "Doe")]

        for user, name in zip(seeded_users, names):
            profile_data = {
                "user_id": user.id,
                "name": name[0],
                "surname": name[1],
            }
            user_profile = UserProfile(**profile_data)
            user_profiles.append(user_profile)

        # Seeding user profiles
        self.seed(user_profiles)



from locust import HttpUser, TaskSet, task
from splent_framework.locust.common import get_csrf_token, fake
from splent_framework.environment.host import get_host_for_locust_testing


class SignupBehavior(TaskSet):
    def on_start(self):
        self.signup()

    @task
    def signup(self):
        response = self.client.get("/signup")
        csrf_token = get_csrf_token(response)

        response = self.client.post(
            "/signup",
            data={
                "email": fake.email(),
                "password": fake.password(),
                "csrf_token": csrf_token,
            },
        )
        if response.status_code != 200:
            print(f"Signup failed: {response.status_code}")


class LoginBehavior(TaskSet):
    def on_start(self):
        self.ensure_logged_out()
        self.login()

    @task
    def ensure_logged_out(self):
        response = self.client.get("/logout")
        if response.status_code != 200:
            print(
                f"Logout failed or no active session: {response.status_code}"
            )

    @task
    def login(self):
        response = self.client.get("/login")
        if response.status_code != 200 or "Login" not in response.text:
            print(
                "Already logged in or unexpected response, redirecting to logout"
            )
            self.ensure_logged_out()
            response = self.client.get("/login")

        csrf_token = get_csrf_token(response)

        response = self.client.post(
            "/login",
            data={
                "email": "user1@example.com",
                "password": "1234",
                "csrf_token": csrf_token,
            },
        )
        if response.status_code != 200:
            print(f"Login failed: {response.status_code}")


class AuthUser(HttpUser):
    tasks = [SignupBehavior, LoginBehavior]
    min_wait = 5000
    max_wait = 9000
    host = get_host_for_locust_testing()



from flask_login import current_user
from splent_framework.decorators.decorators import pass_or_abort


def guest_required(f):

    def condition(**kwargs):
        return not current_user.is_authenticated

    return pass_or_abort(condition)(f)



from flask import url_for
import pytest


@pytest.fixture(scope="function")
def auth_test_client(test_client):
    """
    Extends the test_client fixture to add additional specific data for module testing.
    """
    with test_client.application.app_context():
        # Add HERE new elements to the database that you want to exist in the test context.
        # DO NOT FORGET to use db.session.add(<element>) and db.session.commit() to save the data.
        pass

    yield test_client

def test_login_success(auth_test_client, auth_test_user):
    response = auth_test_client.post("/login", data=dict(
        email="user1@example.com", password="1234"
    ), follow_redirects=True)

    assert b"Invalid credentials" not in response.data


def test_login_unsuccessful_bad_email(auth_test_client):
    response = auth_test_client.post(
        "/login",
        data=dict(email="bademail@example.com", password="test1234"),
        follow_redirects=True,
    )

    assert response.request.path == url_for("auth.login"), "Login was unsuccessful"
    auth_test_client.get("/logout", follow_redirects=True)


def test_login_unsuccessful_bad_password(auth_test_client):
    response = auth_test_client.post(
        "/login",
        data=dict(email="user1@example.com", password="wrongpass"),
        follow_redirects=True,
    )

    assert b"Invalid credentials" in response.data



"""splent_feature_auth

Revision ID: 8c02a7f8902b
Revises: 742c1ee13917
Create Date: 2026-03-22 23:10:23.192149

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '8c02a7f8902b'
down_revision = '742c1ee13917'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



"""splent_feature_auth

Revision ID: 742c1ee13917
Revises: fd0b989746d8
Create Date: 2026-03-21 23:56:23.736797

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '742c1ee13917'
down_revision = 'fd0b989746d8'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
